# web interface to a bazaar repository
#
# Copyright 2006 Goffredo Baroncelli <kreijack@inwind.it>
#
# This file is part of bazaar-webserve.
#
# bazaar-webserve is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# bazaar-webserve is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with bazaar-webserve; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import bzrlib
from bzrlib.option import Option
import test_webserve

class cmd_webserve(bzrlib.commands.Command):
    """Start the 'webserve' web interface"""

    takes_options = [
        Option('templates', type=str),
        Option('address', type=str),
        Option('port', type=int),
        Option('ipv6'),
        Option('acceslog', type=str),
        Option('errorlog', type=str),
        Option('lock'),
        Option('profile'),
        Option('tararchive'),
        Option('style', type=str),
        Option('fileserver'),
        Option('commentfilter', type=str),
        Option('encoding', type=str),
    ]
    takes_args = ['name?', 'rootrepository?']

    def run(self, name = None, rootrepository = None,
            templates = None, address = "", port = 8088, ipv6 = None,
            accesslog = '', errorlog = '', lock=None, profile=None,
            tararchive=None, style="", fileserver=None, commentfilter=None,
            encoding=None):

        import hgweb
        import os

        if rootrepository == None:
            rootrepository = os.getcwd()
        if name == None:
            name = os.path.basename(rootrepository)
        #if templates == None:
        #    templates = os.path.join( rootrepository, "templates")

        if style:
            style = "map-"+style
        else:
            style = "map"

        httpd = hgweb.create_server(rootrepository, name, templates,
                 address, port, ipv6, accesslog, errorlog,
                 lock, tararchive, profile, style, fileserver,
                 commentfilter and commentfilter.split(";;") or [],
                 encoding)
        httpd.serve_forever()


bzrlib.commands.register_command(cmd_webserve)
bzrlib.tests.MODULES_TO_TEST.append(test_webserve)


